// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibuild;

/// Message property information
class OmnibuildProperty {
  /// Property name
  final String name;

  /// Property docomentation
  final String documentation;

  /// Property type name
  final String type;

  /// PDS type
  final String pdsType;

  /// Indicate if the property is an enumeration.
  final bool enumeration;

  /// Indicate if the property is passed as named argument.
  final bool named;

  /// Indicate the default value of the property.
  final String defaultValue;

  /// Constructor
  OmnibuildProperty(this.name, this.documentation, this.type, this.pdsType,
      this.enumeration, this.named, this.defaultValue);
}

/// Message information
class OmnibuildMessage {
  /// Full omnibusId
  final String omnibusId;

  /// Message name
  final String name;

  /// Message documentation
  final String documentation;

  /// Message type (event, request, response)
  final OmnibusMessageType type;

  /// Message properties
  final List<OmnibuildProperty> properties;

  /// Indices of filter properties in [properties]
  final List<OmnibuildProperty> filter;

  /// Constructor
  OmnibuildMessage(this.omnibusId, this.name, this.documentation, this.type,
      this.properties, this.filter);

  /// Get super class (OmnibusEvent, OmnibusRequest or OmnibusResponse).
  String get superClass => type == OmnibusMessageType.event
      ? 'OmnibusEvent'
      : (type == OmnibusMessageType.request
          ? 'OmnibusRequest'
          : 'OmnibusResponse');
}

/// Message specification information
class OmnibuildSpecFile {
  /// Major version
  final int version;

  /// Parent package
  final String package;

  /// Copyright
  String copyright;

  /// Parent library
  String library;

  /// Description
  String description;

  /// Messages
  List<OmnibuildMessage> messages = new List<OmnibuildMessage>();

  /// Construct from YAML data.
  OmnibuildSpecFile.fromYaml(String yamlString, this.version, this.package) {
    // Should pass by reference.
    _parseYaml(this, yamlString);
  }

  /// Generate code.
  String generateCode() {
    var code =
        _omnibuildTemplate.renderString({'data': this, 'fn': _fnLambdas});

    // Generate formatted code.
    //
    // Note: an exception will be raised if the user specifies illegal types or
    // default values in the specification. In this case the code will fail and
    // the exception will propagate to higher-level mechanisms.
    var formatter = new DartFormatter();

    // Return generated code.
    return formatter.format(code.toString());
  }
}
