// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibuild.test.class_test;

import 'package:test/test.dart';

import 'data/message_library.dart' as v0;

void main() {
  test('Default value initialization', () {
    var dflt = new v0.ExoticDefaults(new Set<List<Set<int>>>());
    expect(dflt.defaultedConstructor.x, equals(1));
    expect(dflt.defaultedList.length, equals(5));
  });

  test('Reverse construction with Point', () {
    var d = new v0.ExoticDefaults(new Set<List<Set<int>>>());
    var dr = new v0.ExoticDefaults.fromPds(d.toPds());
    expect(dr.defaultedConstructor.x, equals(1));
  });

  test('Reverse construction with List<List<Point>> and Map<int, String>', () {
    var d = new v0.ImpureMessage([], [
      [new v0.Point(1, 2), new v0.Point(3, 4)]
    ], {
      0: 'zero',
      1: 'one'
    });
    var dr = new v0.ImpureMessage.fromPds(d.toPds());
    expect(dr.impureList[0][1].x, equals(3));
    expect(dr.intMap[0], equals('zero'));
  });

  test('Reverse construction with Map<String, List<Map<String, int>>>', () {
    var d = new v0.PureMessage([], {
      'a': [
        {'b': 3}
      ]
    });
    var dr = new v0.PureMessage.fromPds(d.toPds());
    expect(dr.pureMap['a'][0]['b'], equals(3));
  });

  test('Reverse construction with enumeration', () {
    var d = new v0.MyMessage();
    var dr = new v0.MyMessage.fromPds(d.toPds());
    expect(dr.impactFactor, equals(v0.ImpactFactor.high));
  });

  // Check reverse construction error throwing.
  test('Reverse construct MyMessage with corrupt data', () {
    // Construct with String instead of integer.
    void construct1() {
      new v0.MyMessage.fromPds({'impactFactor': 'high'});
    }

    // Construct with empty map.
    void construct2() {
      new v0.MyMessage.fromPds({});
    }

    expect(construct1, throwsArgumentError);
    expect(construct2, throwsArgumentError);
  });

  test('Reverse construct a Set<List<Set<int>>>', () {
    var d = new v0.ExoticDefaults(new Set<List<Set<int>>>.from([
      [
        new Set<int>.from([1, 3, 5])
      ]
    ]));
    var dr = new v0.ExoticDefaults.fromPds(d.toPds());
    expect(dr.defaultedSet.toList()[0][0].contains(3), equals(true));
  });
}
